from datetime import datetime

from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.service import DiscordBotServiceABC
from discord import RawReactionActionEvent

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.helper.log_message_helper import LogMessageHelper
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.configuration.base_server_settings import BaseServerSettings


class BaseReactionHandler:
    def __init__(
        self,
        logger: LoggerABC,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        base_helper: BaseHelperABC,
        client_utils: ClientUtilsABC,
        db: DatabaseContextABC,
    ):
        self._logger = logger
        self._bot = bot
        self._servers = servers
        self._users = users
        self._base_helper = base_helper
        self._client_utils = client_utils
        self._db = db

    async def handle(self, payload: RawReactionActionEvent, r_type=None) -> None:
        self._logger.trace(__name__, f"Handle reaction {payload} {r_type}")

        guild = self._bot.get_guild(payload.guild_id)
        member = guild.get_member(payload.user_id)
        if member is None:
            self._logger.warn(__name__, f"User {payload.user_id} in {guild.name} not found - skipping")
            return

        try:
            log_msg = f"{member.name} reacted"
            if payload.emoji.name is not None:
                log_msg += f" with {payload.emoji.name}"
            try:
                channel = guild.get_channel(payload.channel_id)
                message = await channel.fetch_message(payload.message_id)
                self._logger.info(
                    __name__,
                    f"{log_msg} to message {LogMessageHelper.get_log_string(message)}",
                )
            except Exception as e:
                self._logger.error(__name__, f"Getting message for reaction logging failed", e)
                self._logger.info(__name__, f"{log_msg} to message {payload.message_id}")
        except Exception as e:
            self._logger.error(__name__, f"Reaction logging failed", e)

        if member.bot:
            return

        server = self._servers.get_server_by_discord_id(guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)
        settings: BaseServerSettings = self._base_helper.get_config(guild.id)

        if r_type == "add":
            if self._client_utils.is_message_xp_count_by_hour_higher_that_max_message_count_per_hour(
                datetime.now(), user, settings, is_reaction=True
            ):
                return

            user.xp += settings.xp_per_reaction
            self._users.update_user(user)
            self._db.save_changes()
        elif r_type == "remove":
            user.xp -= settings.xp_per_reaction
            self._users.update_user(user)
            self._db.save_changes()
        else:
            self._logger.warn(__name__, f"Invalid reaction type {r_type}")
